package LWP::Protocol::proxyproto::Socket::Base;

use strict;
use warnings;

our $VERSION = '0.01'; # VERSION

use IO::Socket;
use Net::IP;
use Scalar::Util qw(looks_like_number);

sub new {
    my $class = shift;
    my %args = @_;
    my $conn = $class->_proxy_connect( \%args );
    bless $conn, $class;
    
    $conn->http_configure( \%args );
    return $conn;
}

sub _proxy_connect {
    my ( $class, $args ) = @_;
    my $agent = $args->{Agent};

    ## Override Proxy Port to use whatever is supplied (or the default) for the connection we're making.
    $args->{'ProxyPort'} = $args->{'PeerPort'};

    ## This is a bit messy, but seems less bad than poking our own data into arbirary parts of the module. 
    my $pp = $agent->{'basic_authentication'}->{'proxyproto'};


    if (!looks_like_number($pp->{'source'}[1]) || $pp->{'source'}[1] < 0 || $pp->{'source'}[1] > 65535) {
        Carp::croak("Proxy protocol source port not valid!");
    }
    if (!looks_like_number($pp->{'dest'}[1]) || $pp->{'dest'}[1] < 0 || $pp->{'dest'}[1] > 65535){
        Carp::croak("Proxy protocol dest port not valid!");
    }
    
    my $sourceip = new Net::IP ($pp->{'source'}[0]) or Carp::croak(Net::IP::Error());
    my $destip = new Net::IP ($pp->{'dest'}[0]) or Carp::croak(Net::IP::Error());

    if ($destip->version() != $sourceip->version()) {
        Carp::croak("Proxy protocol source and dest IP versions do not match!");
    }

    my $protostring = sprintf("PROXY TCP%d %s %s %d %d\r\n", $destip->version(),
                        $pp->{'source'}[0],$pp->{'dest'}[0],
                        $pp->{'source'}[1],$pp->{'dest'}[1]);

    my $conn = IO::Socket::IP->new(PeerAddr => $args->{'ProxyAddr'},
                                    PeerPort => $args->{'ProxyPort'},
                                    Proto    => "tcp",
                                    Type     => SOCK_STREAM)
        or Carp::croak("Couldn't connect to $args->{'ProxyAddr'}:$args->{'ProxyPort'} : $@\n");

    print $conn $protostring; 

    delete $args->{ProxyAddr};
    delete $args->{ProxyPort};
    delete $args->{ProxyUserinfo};
    delete $args->{Agent};

    return( $conn );
}

sub http_connect {
    return 1;
}

1;
